import random
import math
import threading
import time

""" """

class ThreadingTest(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.setDaemon(True)
        self.i = 0

    def run(self):
        print "Start."
        while True:
            time.sleep(10)
            self.i += 1
            print self.i * 10


""" Tester """

class SimpleMCMCSimulator:
    def __init__(self,teststatistics,mcmcsampler):
        self.mcmcsampler=mcmcsampler
        self.teststatistics=teststatistics
        self.obs=self.teststatistics.getObservedValue()
        self.initializeData()

    def initializeData(self):
        self.num=0
        self.sig=0

    def getSimulatedPValue(self):
        return float(self.sig)/float(self.num)

    def runSimulation(self,length,burnin=0,printflag=False, printinterval=1000):
        sig=0
        i=0
        for x in self.mcmcsampler.xRandomElements(burnin):
            if printflag:
                i=i+1
                if i % printinterval == 0:
                    print self.teststatistics.getValue(x)
        for x in self.mcmcsampler.xRandomElements(length):
            t=self.teststatistics.getValue(x)
            if t>=self.obs:
                sig=sig+1
                self.sig=self.sig+1
            self.num=self.num+1
            if printflag:
                i=i+1
                if i % printinterval == 0:
                    print t

        return float(sig)/float(length)
        
    

""" Test statistics """
class TestStatisticsTools:
    @classmethod
    def getAtehamechi(self,xo,subtables,eps):
        I=xo.keys()
        S=[(Si, [i for i in I if not i in Si]) for Si in subtables]
        b=[(sum(xo[i] for i in Si),sum(xo[i] for i in cSi)) for (Si,cSi) in S]
        r={}
        for i in I:
            r[i]=1.0
        dist=eps*2
        while dist>eps:
            for ((Si,cSi),(bi,cbi)) in zip(S,b):
                tot=sum(r[i] for i in Si)
                for i in Si:
                    if tot!=0:
                        r[i]=r[i]*bi/tot
                tot=sum(r[i] for i in cSi)
                for i in cSi:
                    if tot!=0:
                        r[i]=r[i]*cbi/tot
            dist=0.0
            for ((Si,cSi),(bi,cbi)) in zip(S,b):
                s1=(sum(r[i] for i in Si)-bi)
                s2=(sum(r[i] for i in cSi)-cbi)
                dist=dist+s1*s1+s2*s2
        return r


class ChisqTestStatistics:
    def __init__(self,observedtable,subtables,eps=0.00000001):
        self.eps=eps
        self.atehamechi=TestStatisticsTools.getAtehamechi(observedtable,subtables,self.eps)
        self.obs=self.getValue(observedtable)
        
    def getObservedValue(self):
        return self.obs

    def getValue(self,x):
        r=0
        for k in x.keys():
            if self.atehamechi[k]!=0:
                r=r+(x[k]-self.atehamechi[k])*(x[k]-self.atehamechi[k])/self.atehamechi[k]
        return r

class GTestStatistics:
    def __init__(self,observedtable,subtablesforH0,subtablesforH1,eps=0.00000001):
        self.eps=eps
        self.atehamechi=TestStatisticsTools.getAtehamechi(observedtable,subtablesforH0,self.eps)
        self.subtableforH1=subtablesforH1
        self.obs=self.getValue(observedtable)
        
    def getObservedValue(self):
        return self.obs

    def getValue(self,x):
        r=0
        mm=TestStatisticsTools.getAtehamechi(x,self.subtableforH1,self.eps)
        for k in x.keys():
            if self.atehamechi[k]!=0 and mm[k]:
                print mm[k]
                r=r+x[k]*math.log(mm[k]/self.atehamechi[k])
        return r


""" Prob Distribution """

class TakoBumpu:
    def __init__(self):
        pass
    
    def ratioOfProb(self,oldx,move):
        """ p(new)/p(old) """
        r=1.0
        for k in move.keys():
            xk=oldx[k]
            if move[k] > 0:
                for i in range(move[k]):
                    r=r/(xk+1+i)
            elif move[k] < 0:
                for i in range(move[k]):
                    r=r*(xk-i)
        return r

class IchiyoBumpu:
    def __init__(self):
        pass
    
    def ratioOfProb(self,oldx,move):
        """ p(new)/p(old) """
        return 1.0



""" MCMC sampler """
class AbstractMCMCSampler:
    def __init__(self,seedelm):
        self.x=seedelm
        self.n_rejected_by_condition=0
        self.n_rejected=0
        self.n_stable=0
        self.n_moved=0

    def getRatioOfProb(self,x,move):
        """ Fix me to return p(new)/p(old). """
        pass

    def getRandomMove(self):
        """ Fix me to return move at random. """
        pass

    def isAdmissibleMove(self,move):
        """ Fix me to return False if move is not admissible. """
        return True

    def getAdmissibleMoveAtRandom(self):
        move=self.getRandomMove()
        while not self.isAdmissibleMove(move):
            self.n_rejected_by_condition=self.n_rejected_by_condition+1
            move=self.getRandomMove()
        return move

    def markovstep(self):
        move=self.getAdmissibleMoveAtRandom()
        for k in move.keys():
            if self.x[k]+move[k] < 0:
                self.n_rejected=self.n_rejected+1
                return 
        q=random.random()
        p=self.getRatioOfProb(self.x,move)
        if p<q:
            self.n_stable=self.n_stable+1
            return 
        for k in move.keys():
            self.x[k]=self.x[k]+move[k]
        self.n_moved=self.n_moved+1
        return 

    def burnin(self,length):
        for i in xrange(length):
            self.markovstep()

    def getRandomElement(self):
        self.markovstep()
        return self.x.copy()

    def xRandomElements(self,length):
        for i in xrange(length):
            self.markovstep()
            yield self.x.copy()

    def getRejectionData(self):
        return (self.n_rejected_by_condition,self.n_rejected, self.n_stable, self.n_moved)

class MCMCSamplerWithMarkovBasis(AbstractMCMCSampler):
    def __init__(self,seedelm,markovbasis,bumpu,condition=None):
        AbstractMCMCSampler.__init__(self,seedelm.copy())
        self.markovbasis=markovbasis
        self.bumpu=bumpu
        self.condition=condition

    def getRatioOfProb(self,x,move):
        return self.bumpu.ratioOfProb(x,move)

    def isAdmissibleMove(self,move):
        if self.condition:
            return self.condition.isAdmissibleMove(move)
        else:
            return True

    def getRandomMove(self):
        move=self.markovbasis.getRandomMove()
        sgn=random.randint(0,1)
        if sgn==1:
            for k in move.keys():
                move[k]=-move[k]
        return move

class MCMCSamplerWithLatticeBasis(AbstractMCMCSampler):
    def __init__(self,seedelm,latticebasis,integervectors,bumpu,condition):
        AbstractMCMCSampler.__init__(self,seedelm.copy())
        self.latticebasis=latticebasis
        self.integervectors=integervectors
        self.bumpu=bumpu
        self.condition=condition

    def getRatioOfProb(self,x,move):
        return self.bumpu.ratioOfProb(x,move)

    def isAdmissibleMove(self,move):
        if self.condition:
            return self.condition.isAdmissibleMove(move)
        else:
            return True

    def getRandomMove(self):
        moves=self.latticebasis.getSomeMovesAtRandom()
        coefs=self.integervectors.getIntegerVectorAtRandom(len(moves))
        move={}
        for mi,ci in zip(moves,coefs):
            for k in mi.keys():
                if not k in move:
                    move[k]=0
                move[k]=move[k]+ci*mi[k]
        return move



""" Condition for Moves """

class ConditionForSubtableModel:
    def __init__(self,subtables):
        self.subtables=subtables

    def isAdmissibleMove(self,move):
        for s in self.subtables:
            amount=sum(move[k] for k in move.keys() if k in s)
            if amount != 0:
                return False
        return True

""" Random generator for Interger vector """

class SimpleIntegerVectors:
    def __init__(self,mu,sigma):
        self.mu=mu
        self.sigma=sigma

    def getRandomInteger(self):
        r=random.gauss(self.mu, self.sigma)
        if r>0:
            r=round(r+0.5)
        elif r<0:
            r=round(r-0.5)
        return r

    def getIntegerVectorAtRandom(self,length):
        return [ self.getRandomInteger() for i in range(length) ]


""" Random generator for Markov Basis """

class UnionOfMarkovBases:
    def __init__(self,markovbases,prob=None):
        self.markovbases=markovbases
        if prob==None:
            self.prob=[1.0/(len(markovbases)) for mi in markovbases]
        else:
            self.prob=prob

    def getRandomMove(self):
        q=random.random()
        for pi,mi in zip(self.prob,self.markovbases):
            if q < pi:
                return mi.getRandomMove()
            else:
                q=q-pi
        return self.markovbases[-1].getRandomMove()

class TransposeOfMarkovBasis:
    def __init__(self,markovbasis,prob=1.0):
        self.markovbasis=markovbasis
        self.prob=prob

    def getRandomMove(self):
        q=random.random()
        m=self.markovbasis.getRandomMove()
        if q<self.prob:
            r={}
            for (i,j) in m.keys():
                r[j,i]=m[i,j]
            return r
        else:
            return m

class RenamedMarkovBasis:
    def __init__(self):
        self.mb=None

    def setMarkovBasis(self,mb):
        self.mb=mb

    def getRandomMove(self):
        return self.mb.getRandomMove()


class BasicMoves:
    def __init__(self,rowindeces,colindeces):
        self.rows=rowindeces
        self.cols=colindeces

    def getRandomMove(self):
        (i1,i2)=random.sample(self.rows,2)
        (j1,j2)=random.sample(self.cols,2)
        r={}
        r[i1,j1]=1
        r[i1,j2]=-1
        r[i2,j1]=-1
        r[i2,j2]=1
        return r


class BasicMovesFixingSubtables:
    def __init__(self,rowindeces,colindeces,subtables):
        self.basicmoves=BasicMoves(rowindeces,colindeces)
        self.subtables=subtables

    def getRandomMove(self):
        move=self.basicmoves.getRandomMove()
        while not self.isadmissible(move):
            move=self.basicmoves.getRandomMove()
        return move

    def isadmissible(self,move):
        for s in self.subtables:
            amount=sum(move[k] for k in move.keys() if k in s)
            if amount != 0:
                return False
        return True


class OgawaMarkovBasisForBlockDiagnal(RenamedMarkovBasis):
    def __init__(self,rows,cols,diagonalblocks,p_type1=0.25,p_type2=0.25,p_type3=0.25):
        subtablesofblocks=[ [p for bi in diagonalblocks for p in bi] ] 
        om1=BasicMovesFixingSubtables(rows,cols,subtablesofblocks)
        om2=OgawaMoveTypeIIForBlockDiagonal(diagonalblocks)
        om3=OgawaMoveTypeIIIForBlockDiagonal(diagonalblocks)
        om4=OgawaMoveTypeIVForBlockDiagonal(rows,cols,diagonalblocks)
        mb=UnionOfMarkovBases([om1,om2,om3,om4],[p_type1,p_type2,p_type3,1.0-p_type1-p_type2,p_type3])
        self.setMarkovBasis(mb)
        

class OgawaMoveTypeIIForBlockDiagonal:
    def __init__(self,diagonalblocks):
        self.diagonalblocks=diagonalblocks

    def getRandomMove(self):
        b=random.sample(self.diagonalblocks,3)
        (i1,j1)=random.choice(b[0])
        (i2,j2)=random.choice(b[1])
        (i3,j3)=random.choice(b[2])
        r={}
#        r[(i1,j1)]=0
        r[(i1,j2)]=1
        r[(i1,j3)]=-1
        r[(i2,j1)]=-1
#        r[(i2,j2)]=0
        r[(i2,j3)]=1
        r[(i3,j1)]=1
        r[(i3,j2)]=-1
#        r[(i3,j3)]=0
        return r



class OgawaMoveTypeIIIForBlockDiagonal:
    def __init__(self,diagonalblocks):
        self.diagonalblocks=diagonalblocks

    def getRandomMove(self):
        b=random.sample(self.diagonalblocks,3)
        (i1,j1)=random.choice(b[0])
        (i2,j2)=random.choice(b[1])
        (i3,j3)=random.choice(b[2])
        r={}
        r[(i1,j1)]=1
#        r[(i1,j2)]=0
        r[(i1,j3)]=-1
#        r[(i2,j1)]=0
        r[(i2,j2)]=-1
        r[(i2,j3)]=1
        r[(i3,j1)]=-1
        r[(i3,j2)]=1
#        r[(i3,j3)]=0

        return r


class OgawaMoveTypeIVForBlockDiagonal(RenamedMarkovBasis):
    def __init__(self,rows,cols,diagonalblocks):
        om4h=OgawaMoveTypeIVhForBlockDiagonal(cols,rows,diagonalblocks)
        mb=TransposeOfMarkovBasis(om4h,0.5)
        self.setMarkovBasis(mb)

class OgawaMoveTypeIVhForBlockDiagonal:
    def __init__(self,rowindeces,colindeces,diagonalblocks):
        self.diagonalblocks=diagonalblocks
        self.ind_blocks=range(len(diagonalblocks))
        rowblocks=[[i for (i,j) in bk] for bk in diagonalblocks]
        colblocks=[[j for (i,j) in bk] for bk in diagonalblocks]
        self.rowblocks=[[i for i in rowindeces if i in bk] for bk in rowblocks]
        self.colblocks=[[j for j in colindeces if j in bk] for bk in colblocks]
        self.rowcomplementblocks=[[i for i in rowindeces if not i in bk] for bk in self.rowblocks]
        self.colcomplementblocks=[[j for j in colindeces if not j in bk] for bk in self.colblocks]


    def getRandomMove(self):
        k=random.sample(self.ind_blocks,2)
        a=k[0]
        b=k[1]
        complementcols=[j for j in self.colcomplementblocks[a] if not j in self.colblocks[b]]

        (i1,j1)=random.choice(self.diagonalblocks[a])
        i2=random.choice(self.rowblocks[a])
        j3=random.choice(complementcols)
        (i3,j2)=random.choice(self.diagonalblocks[b])
        i4=random.choice(self.rowblocks[b])
        j4=random.choice(complementcols)


        r={}
        for i in [i1,i2,i3,i4]:
            for j in [j1,j2,j3,j4]:
                r[i,j]=0
        r[i1,j1]=r[i1,j1]+1
        r[i1,j3]=r[i1,j3]-1
        r[i2,j2]=r[i2,j2]+1
        r[i2,j4]=r[i2,j4]-1
        r[i3,j2]=r[i3,j2]-1
        r[i3,j3]=r[i3,j3]+1
        r[i3,j1]=r[i3,j1]-1
        r[i3,j4]=r[i3,j4]+1
        return r

""" Random generator for Lattice Basis """
class LatticeBasisFromBaseGeneratorByGeometricDistribution:
    def __init__(self,basegenerator,prob):
        self.basegenerator=basegenerator
        self.prob=prob
    def getSomeMovesAtRandom(self):
        r=[self.basegenerator.getRandomMove()]
        while(random.random()<prob):
            r.append(self.basegenerator.getRandomMove())
        return r
